@@ -27,15 +27,24 @@ from cirruscluster.ext.nx import password
 from boto.ec2 import connection as ec2_connection 
 from boto.iam import connection as iam_connection
 from boto.s3 import connection as s3_connection
+from boto.s3 import key as s3_key
 from cirruscluster import core
 import pkg_resources 
 import string
-import hashlib
 import logging
 import time
 
+cirrus_iam_username = 'cirrus'
 workstation_profile = 'cirrus_workstation_profile'
 
+class UnsupportedAwsRegion(Exception):
+  pass
+
+
+class InvalidAwsCredentials(Exception):
+  pass
+
+
 class InstanceInfo(object):
   def __init__(self, name, id, state, hostname):
     self.name = name
@@ -44,48 +53,109 @@ class InstanceInfo(object):
     self.hostname = hostname
     return
 
-def IAMUserReady(iam_aws_id, iam_aws_secret):
-  """ Returns true if IAM user can login. """
-  ready = False
-  if iam_aws_id and iam_aws_secret:
-    try:
-      # test that ec2 connection works
-      test_ec2 = ec2_connection.EC2Connection(iam_aws_id, iam_aws_secret)
-      test_ec2.get_all_instances()
-      ready = True
-    except:
-      logging.info( 'failed to connect as user: %s' % (iam_aws_id))
-      
-  return ready
+  
 
 
-def InitCirrusIAMUser(root_aws_id, root_aws_secret):
-  """ Configure cirrus IAM user.
-  Given a user's root aws credentials, setup the IAM user environment that
-  can be used later for all future api actions.
-  """
-  #iam = iam_connection.IAMConnection(root_aws_id, root_aws_secret)
-  #s3 = s3_connection.S3Connection(root_aws_id, root_aws_secret)
-  
-  iam = core.CreateTestedIamConnection(root_aws_id, root_aws_secret)
-  s3 = core.CreateTestedS3Connection(root_aws_id, root_aws_secret)
-  
-  
-  cirrus_iam_username = 'cirrus'
-  bucketname = 'cirrus_iam_config_%s' % (hashlib.md5(root_aws_id).hexdigest())
-  has_cirrus_user = False
-  response = iam.get_all_users()
-  for user in response['list_users_response']['list_users_result']['users']:
-    logging.info( 'iam user exists: %s' % user)
-    has_cirrus_user = True
-  if not has_cirrus_user:
-    logging.info( 'creating iam user')
-    response = iam.create_user(cirrus_iam_username)
-    # setup role so workstation can assume IAM credentials without additional
-    # configuration
-    logging.info( 'Created iam role and policy')
-    role_name = 'cirrus_workstation'
+
+def CirrusIamUserReady(iam_aws_id, iam_aws_secret):
+  """ Returns true if provided IAM credentials are ready to use. """
+  is_ready = False
+  try:
+    s3 = core.CreateTestedS3Connection(iam_aws_id, iam_aws_secret)
+    if s3:
+      if core.CirrusAccessIdMetadata(s3, iam_aws_id).IsInitialized():
+        is_ready = True            
+  except boto.exception.BotoServerError as e:
+    print e 
     
+  return is_ready
+
+
+def GetCirrusIamUserCredentials(root_aws_id, root_aws_secret):
+  return CirrusIamUserManager(root_aws_id, root_aws_secret).GetCredentials()
+  
+
+def NoSuchEntityOk(f):
+  """ Decorator to remove NoSuchEntity exceptions, and raises all others. """
+  def ExceptionFilter(*args):
+    try:
+      return f(*args)
+    except boto.exception.BotoServerError as e:
+      if e.error_code == 'NoSuchEntity':
+        pass
+      else:
+        raise
+    except:  
+      raise
+    return False
+  return ExceptionFilter
+  
+  
+class CirrusIamUserManager(object):
+  """ Creates the IAM user and the required policies. """
+  def __init__(self, root_aws_id, root_aws_secret):
+    self.root_aws_id = root_aws_id
+    self.root_aws_secret = root_aws_secret
+    self.iam = core.CreateTestedIamConnection(root_aws_id, root_aws_secret)
+    if not self.iam:
+      raise InvalidAwsCredentials()
+    self.s3 = core.CreateTestedS3Connection(root_aws_id, root_aws_secret)
+    if not self.s3:
+      raise InvalidAwsCredentials()    
+    
+    self.role_name = 'cirrus_workstation'    
+    return
+    
+  def GetCredentials(self):
+    if not self.__IsInitialized():
+      self.__Create()      
+    iam_id = self.GetAccessKeyId()
+    iam_secret = core.CirrusAccessIdMetadata(self.s3, iam_id).GetSecret()
+    return iam_id, iam_secret
+    
+  def __IsInitialized(self):      
+    """ Returns true if IAM user initialization has completed. """  
+    is_initialized = False
+    iam_id = self.GetAccessKeyId()
+    if iam_id:
+      if core.CirrusAccessIdMetadata(self.s3, iam_id).IsInitialized():
+        is_initialized = True
+    return is_initialized
+  
+  def __Cleanup(self):
+    logging.info('Cleaning up iam user...')
+    NoSuchEntityOk(self.iam.remove_role_from_instance_profile)(workstation_profile, self.role_name)
+    NoSuchEntityOk(self.iam.delete_instance_profile)(workstation_profile)
+    role_policy_names = []
+    response = NoSuchEntityOk(self.iam.list_role_policies)(self.role_name)
+    if response:
+      role_policy_names = response['list_role_policies_response'] \
+                                  ['list_role_policies_result']['policy_names']                                        
+      for role_policy_name in role_policy_names:
+        self.iam.delete_role_policy(self.role_name, role_policy_name)
+    NoSuchEntityOk(self.iam.delete_role)(self.role_name)
+    user_policy_names = []    
+    response = NoSuchEntityOk(self.iam.get_all_user_policies)(cirrus_iam_username)
+    if response:
+      user_policy_names = response['list_user_policies_response'] \
+                                  ['list_user_policies_result']['policy_names']
+      for user_policy_name in user_policy_names:
+        self.iam.delete_user_policy(cirrus_iam_username, user_policy_name)
+    response = NoSuchEntityOk(self.iam.get_all_access_keys)(cirrus_iam_username)
+    if response:
+      access_keys = response['list_access_keys_response'] \
+                            ['list_access_keys_result']['access_key_metadata']
+      for access_key in access_keys:
+        self.iam.delete_access_key(access_key['access_key_id'], 
+                                   access_key['user_name'])                             
+    NoSuchEntityOk(self.iam.delete_user)(cirrus_iam_username)
+    return
+  
+  def __Create(self):
+    self.__Cleanup()    
+    response = self.iam.create_user(cirrus_iam_username)
+    # setup role so workstation can assume IAM credentials without additional
+    # configuration    
     power_user_policy_json = """{
         "Version": "2012-10-17",
         "Statement": [
@@ -96,95 +166,63 @@ def InitCirrusIAMUser(root_aws_id, root_aws_secret):
             }
         ]
     }"""
+    self.iam.put_user_policy(cirrus_iam_username, 'power_user_policy', power_user_policy_json)
     # ensure no conflicting role exists with same name
-    try:
-      iam.remove_role_from_instance_profile(workstation_profile, role_name)
-    except:
-      pass
-    try:
-      iam.delete_instance_profile(workstation_profile)
-    except:
-      pass
-    response = iam.list_role_policies(role_name)
-    role_policy_names = response['list_role_policies_response'] \
-                                ['list_role_policies_result']['policy_names']
-    for role_policy_name in role_policy_names:
-      iam.delete_role_policy(role_name, role_policy_name)
-    try:
-      iam.delete_role(role_name)
-    except:
-      raise
-    iam.create_instance_profile(workstation_profile)
+    self.iam.create_instance_profile(workstation_profile)
     workstation_role_arn = None
-    try:
-      response = iam.create_role(role_name)
-      workstation_role_arn = response['create_role_response'] \
-                                     ['create_role_result']['role']['arn']
-    except:
-      pass
+    response = self.iam.create_role(self.role_name)
+    workstation_role_arn = response['create_role_response'] \
+                                   ['create_role_result']['role']['arn']
     assert(workstation_role_arn)
-    iam.add_role_to_instance_profile(workstation_profile, role_name)
-    iam.put_role_policy(role_name, 'power_user_policy', power_user_policy_json)
+    self.iam.add_role_to_instance_profile(workstation_profile, self.role_name)
+    self.iam.put_role_policy(self.role_name, 'power_user_policy', power_user_policy_json)
     # update iam user to have right to launch instances with the 
     # cirrus_workstatio_role
-    policy_json = '{"Version": "2012-10-17", ' \
+    assume_role_policy_json = '{"Version": "2012-10-17", ' \
                   '"Statement": [{"Effect": "Allow", "Action":"iam:PassRole",' \
                   ' "Resource": "%s"}]}' % (workstation_role_arn)
-    iam.put_user_policy(cirrus_iam_username, 
-                        'assume_cirrus_workstation_role', policy_json)
-  policy_json = '{"Version": "2012-10-17", "Statement": [{"Effect": "Allow",' \
-                ' "NotAction": "iam:*", "Resource": "*"}]}'
-  iam.put_user_policy(cirrus_iam_username, 'power_user_policy', policy_json)
-  response = iam.get_all_access_keys(cirrus_iam_username)
-  iam_id = None
-  iam_secret = None  
-  for key in response['list_access_keys_response']['list_access_keys_result'] \
-                     ['access_key_metadata']:
-    if key['status'] == 'Active' and key['user_name'] == cirrus_iam_username:
-      iam_id = key['access_key_id']
-      break
-  cred_bucket = s3.lookup(bucketname)
-  if not cred_bucket:
-    cred_bucket = s3.create_bucket(bucketname, policy='private')
-  if iam_id:
-    # fetch secret from s3
-    cirrus_cred_bucket_key = 'cirrus_iam_sec_%s' % \
-     (hashlib.md5(iam_id).hexdigest())
-    key = cred_bucket.lookup(cirrus_cred_bucket_key)
-    if key:
-      iam_secret = key.get_contents_as_string()
-
-  # part of the credentials is unknown, so make new ones and store those
-  if not iam_id or not iam_secret:
+    self.iam.put_user_policy(cirrus_iam_username, 
+                        'assume_cirrus_workstation_role', assume_role_policy_json)
     logging.info('Creating new aws credentials for IAM user %s.'\
                   % cirrus_iam_username)
-    # Delete any existing acces keys (to prevent from exceeding limit of 2)
-    res = iam.get_all_access_keys(cirrus_iam_username)
-    for key_metadata in res['list_access_keys_response']['list_access_keys_result']['access_key_metadata']:
-      iam.delete_access_key(key_metadata.access_key_id, user_name = cirrus_iam_username)
-      
-    response = iam.create_access_key(cirrus_iam_username)
+    response = self.iam.create_access_key(cirrus_iam_username)
     new_key = response['create_access_key_response']\
                       ['create_access_key_result']\
                       ['access_key']
     iam_id = new_key['access_key_id']
     assert(iam_id)
     iam_secret = new_key['secret_access_key']
-    iam_hash = hashlib.md5(iam_id).hexdigest()
-    cirrus_cred_bucket_key = 'cirrus_iam_sec_%s' % (iam_hash)
-    #store secret in s3 for future use
-    k = s3_connection.Key(cred_bucket)
-    k.key = cirrus_cred_bucket_key
-    k.set_contents_from_string(iam_secret)
-  return iam_id, iam_secret
-
-
-
-class UnsupportedAwsRegion(Exception):
-  pass
-
-class InvalidAwsCredentials(Exception):
-  pass
+    metadata = core.CirrusAccessIdMetadata(self.s3, iam_id)
+    metadata.SetSecret(iam_secret)  
+    metadata.SetInitialized()
+    
+    
+    self.__WaitForCredentialsLive(iam_id, iam_secret)
+    return
+  
+  def __WaitForCredentialsLive(self, iam_aws_id, iam_aws_secret):
+    """ Wait until the new credentials are accepted in all regions and for ec2 and boto services.
+       If we don't do this, we may have a race condition where calls fail (and we can't tell if the credentials are bad (stop) or not yet valid (keep trying))
+    """
+    print 'waiting for credentials to go live...'
+    for region in core.tested_region_names:
+      core.WaitForEc2Connection(iam_aws_id, iam_aws_secret, region)
+    core.WaitForS3Connection(iam_aws_id, iam_aws_secret)
+    
+    print 'done'
+    return
+    
+  def GetAccessKeyId(self):
+    access_key_id = None
+    response = NoSuchEntityOk(self.iam.get_all_access_keys)(cirrus_iam_username)
+    if response:
+      for key in response['list_access_keys_response'] \
+                         ['list_access_keys_result'] \
+                         ['access_key_metadata']:
+        if key['status'] == 'Active':
+          access_key_id = key['access_key_id']
+          break
+    return access_key_id
 
 
 class Manager(object):
@@ -194,6 +232,8 @@ class Manager(object):
     if region_name not in core.tested_region_names:
       raise UnsupportedAwsRegion()
     self.region_name = region_name
+    if not CirrusIamUserReady(iam_aws_id, iam_aws_secret):
+      raise InvalidAwsCredentials()
     self.iam_aws_id = iam_aws_id
     self.iam_aws_secret = iam_aws_secret    
     self.ec2 = core.CreateTestedEc2Connection(iam_aws_id, iam_aws_secret, 
@@ -206,63 +246,24 @@ class Manager(object):
     self.workstation_tag = 'cirrus_workstation'
     self.workstation_keypair_name = 'cirrus_workstation'
     self.ssh_key = None
-    tmp_hash = hashlib.md5(iam_aws_id).hexdigest()    
-    config_bucketname = 'cirrus_workstation_config_%s' % tmp_hash      
     src_region = self.region_name
     dst_regions = core.tested_region_names
     self.ssh_key = core.InitKeypair(self.iam_aws_id, self.iam_aws_secret, 
-                                    self.ec2, self.s3, config_bucketname, 
+                                    self.ec2, self.s3,  
                                     self.workstation_keypair_name, src_region,
                                     dst_regions)
     return
 
 
-#  @core.RetryUntilReturnsTrue(tries=5)
-#  def __CreateValidatedConnections(self):
-#    """ Retries in case IAM fails because IAM credentials are new and not yet
-#        propagated to all regions.
-#    """ 
-#    region = core.GetRegion(self.region_name)
-#    # test that ec2 connection works
-#    test_ec2 = ec2_connection.EC2Connection(self.iam_aws_id, 
-#                                            self.iam_aws_secret,
-#                                            region = region)
-#    try:        
-#      test_ec2.get_all_images(owners=['self'])      
-#    except boto.exception.EC2ResponseError as e:
-#      if e.error_code == 'AuthFailure' or e.error_code == 'InvalidAccessKeyId':
-#        print 'ec2 connect failed... will retry...'
-#        return False
-#    except:
-#      raise
-#    
-#    test_s3 = s3_connection.S3Connection(self.iam_aws_id, 
-#                                         self.iam_aws_secret)
-#    try:        
-#      test_s3.get_all_buckets()      
-#    except boto.exception.S3ResponseError as e:
-#      if e.error_code == 'AuthFailure' or e.error_code == 'InvalidAccessKeyId':
-#        print 's3 connect failed... will retry...'
-#        return False
-#    except:
-#      raise    
-#    self.ec2 = test_ec2
-#    self.s3 = test_s3
-#    return True
-
   def Debug(self):
     res = self.ec2.get_all_instances(instance_ids=['i-b65fbbda'])
     instance = res[0].instances[0]
-    
     params = {'InstanceId' : instance.id,
               'BlockDeviceMapping.1.DeviceName' : '/dev/sda1',
               'BlockDeviceMapping.1.Ebs.VolumeId' : 'vol-c3516299',
               'BlockDeviceMapping.1.Ebs.DeleteOnTermination' : 'true'}
     instance.connection.get_status('ModifyInstanceAttribute', params, verb='POST')
     instance.update()
-    
-    
-    
     return
   
   def ListInstances(self):
@@ -283,23 +284,23 @@ class Manager(object):
     return info
 
   def TerminateInstance(self, instance_id):
-      instance = self.__GetInstanceById(instance_id)
-      assert(instance)
-      instance.modify_attribute('disableApiTermination', False)
-      self.ec2.terminate_instances([instance_id])
-      return
+    instance = self.__GetInstanceById(instance_id)
+    assert(instance)
+    instance.modify_attribute('disableApiTermination', False)
+    self.ec2.terminate_instances([instance_id])
+    return
 
   def StopInstance(self, instance_id):
-      instance = self.__GetInstanceById(instance_id)
-      assert(instance)
-      self.ec2.stop_instances([instance_id])
-      return
+    instance = self.__GetInstanceById(instance_id)
+    assert(instance)
+    self.ec2.stop_instances([instance_id])
+    return
     
   def StartInstance(self, instance_id):
-      instance = self.__GetInstanceById(instance_id)
-      if instance.state != 'running':
-        instance.start()
-      return    
+    instance = self.__GetInstanceById(instance_id)
+    if instance.state != 'running':
+      instance.start()
+    return    
 
   def CreateInstance(self, workstation_name, instance_type, ubuntu_release_name,
                      mapr_version, ami_release_name, ami_owner_id):
@@ -397,11 +398,9 @@ class Manager(object):
     core.WaitForVolumeAttached(new_volume)
     snapshot.delete()
     self.ec2.delete_volume(orig_root_volume_id)
-    
     dot_value = 'false'
     if orig_root_volume_termination_setting:
       dot_value = 'true'
-    
     # restore the del on terminate property
     params = {'InstanceId' : instance.id,
               'BlockDeviceMapping.1.DeviceName' : '/dev/sda1',
@@ -409,7 +408,6 @@ class Manager(object):
               'BlockDeviceMapping.1.Ebs.DeleteOnTermination' : dot_value}
     instance.connection.get_status('ModifyInstanceAttribute', params, verb='POST')
     instance.update()
-    
     return
 
   def AddNewVolumeToInstance(self, instance_id, vol_size_gb):
@@ -419,7 +417,6 @@ class Manager(object):
       raise RuntimeError('Adding volumes this large has not been tested.')
     instance = self.__GetInstanceById(instance_id)
     assert(instance)
-
     # select an unused device
     # see http://askubuntu.com/questions/47617/
     # how-to-attach-new-ebs-volume-to-ubuntu-machine-on-aws
